"""
Delay module for time-based audio processing.

This package provides functionality for creating and managing
audio delay effects and processing.
"""

from .core.delay_essence import Delay
from .core.echoes_essence import EchoesPlatform

__version__ = "0.1.0"
